
#ifndef SC_BOOST_MPL_AUX_CONFIG_WORKAROUND_HPP_INCLUDED
#define SC_BOOST_MPL_AUX_CONFIG_WORKAROUND_HPP_INCLUDED

// Copyright Aleksey Gurtovoy 2002-2004
//
// Distributed under the Boost Software License, Version 1.0. 
// (See accompanying file LICENSE_1_0.txt or copy at 
// http://www.boost.org/LICENSE_1_0.txt)
//
// See http://www.boost.org/libs/mpl for documentation.

// $Source: /Users/acg/CVSROOT/systemc-2.3/src/sysc/packages/boost/mpl/aux_/config/workaround.hpp,v $
// $Date: 2009/10/14 19:11:07 $
// $Revision: 1.2 $

#include <sysc/packages/boost/detail/workaround.hpp>

#endif // SC_BOOST_MPL_AUX_CONFIG_WORKAROUND_HPP_INCLUDED
